/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.idm.demo;

import java.security.Principal;
import java.util.Collection;
import java.util.Properties;
import java.util.ResourceBundle;

import org.jboss.security.idm.Identity;
import org.jboss.security.idm.IdentityException;
import org.jboss.security.idm.LoginContext;
import org.jboss.security.idm.LoginProvider;

public class DemoLoginProvider implements LoginProvider 
{
    private String id = null;
    
    private ResourceBundle users = null; 

    /**
     * 
     *
     */
    public DemoLoginProvider() 
    {
        super();
        this.users = ResourceBundle.getBundle("users");
    }
    
    /**
     * 
     */
    public String getId() throws IdentityException 
    {     
        return this.id;
    }
    
    /**
     * Sets the unique id for the LoginProvider
     * 
     * @param id
     * @throws IdentityException
     */
    public void setId(String id) throws IdentityException
    {
       this.id = id; 
    }
    
    /**
     * Sets any properties like LDAP URLs, JDBC configuration, etc needed to configure this particular implementation
     * of the LoginProvider
     * 
     * @param properties
     * @throws IdentityException
     */
    public void setProperties(Properties properties) throws IdentityException
    {
        
    }

    /**
     * 
     */
    public Identity read(Principal principal) throws IdentityException 
    {    
        return this.read(principal.getName());
    }

    /**
     * 
     */
    public Identity read(String username) throws IdentityException 
    {    
        Identity identity = new Identity();
        
        String password = this.users.getString(username);
        identity.setUserName(username);
        identity.setPassword(password.getBytes());
        identity.setActive(true);
        
        return identity;
    }

    /**
     * 
     */
    public boolean exists(Principal principal) throws IdentityException 
    {    
        return this.exists(principal.getName());
    }

    /**
     * 
     */
    public boolean exists(String username) throws IdentityException 
    {    
        return this.read(username)!=null;
    }

    /**
     * 
     */
    public boolean login(Principal principal, byte[] password)
    throws IdentityException 
    {     
        return this.login(principal.getName(),password);
    }

    /**
     * 
     */
    public boolean login(String username, byte[] password)
            throws IdentityException 
    {        
        String storedPassword = new String(this.read(username).getPassword());
        return storedPassword.equals(new String(password));
    }

    /**
     * 
     */
    public Collection readAllRoles() throws IdentityException 
    {        
        return null;
    }

    /**
     * 
     */
    public void processSSOLoginNotification(LoginContext loginContext)
            throws IdentityException 
    {
    }
}
